@@ -57,10 +57,6 @@
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation>Këto janë Bitcoin adresat e juaja për të dërguar pagesa. Gjithmon kontrolloni shumën dhe adresën pranuese para se të dërgoni monedha.</translation>
     </message>
-    <message>
-        <source>These are your Bitcoin addresses for receiving payments. It is recommended to use a new receiving address for each transaction.</source>
-        <translation>Këto janë Bitcoin adresat e juaja për të pranuar pagesa. Rekomandohet që gjithmon të përdorni një adresë të re për çdo transaksion.</translation>
-    </message>
     <message>
         <source>&amp;Copy Address</source>
         <translation>&amp;Kopjo adresen</translation>
@@ -120,8 +116,8 @@
         <translation>Përsërisni fjalëkalimin e ri</translation>
     </message>
     <message>
-        <source>Show password</source>
-        <translation>Trego fjalëkalimin</translation>
+        <source>Show passphrase</source>
+        <translation>Shfaqe fjalëkalimin</translation>
     </message>
     <message>
         <source>Encrypt wallet</source>
@@ -147,10 +143,6 @@
         <source>Change passphrase</source>
         <translation>Ndrysho fjalëkalimin</translation>
     </message>
-    <message>
-        <source>Enter the old passphrase and new passphrase to the wallet.</source>
-        <translation>Fusni fjalëkalimin e vjetër dhe fjalëkalimin e ri që të hapni portofolin.</translation>
-    </message>
     <message>
         <source>Confirm wallet encryption</source>
         <translation>Konfirmoni enkriptimin e portofolit</translation>
@@ -163,6 +155,14 @@
         <source>Wallet encrypted</source>
         <translation>Portofoli u enkriptua</translation>
     </message>
+    <message>
+        <source>Enter the new passphrase for the wallet.&lt;br/&gt;Please use a passphrase of &lt;b&gt;ten or more random characters&lt;/b&gt;, or &lt;b&gt;eight or more words&lt;/b&gt;.</source>
+        <translation>Jepe fjalëkalimin e ri për portofolin. Ju lutemi të përdorni një fjalkalim prej dhjetë ose më shumë shkronjave të rëndomta, ose tetë e më shumë fjalë.</translation>
+    </message>
+    <message>
+        <source>Enter the old passphrase and new passphrase for the wallet.</source>
+        <translation>Jepe fjalëkalimin e vjetër dhe fjalkalimin e ri për portofolin.</translation>
+    </message>
     <message>
         <source>Wallet encryption failed</source>
         <translation>Enkriptimi i portofolit dështoi</translation>
@@ -225,10 +225,6 @@
         <source>Change the passphrase used for wallet encryption</source>
         <translation>Ndrysho frazkalimin e përdorur per enkriptimin e portofolit</translation>
     </message>
-    <message>
-        <source>Bitcoin</source>
-        <translation>Bitcoin</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Dergo</translation>
@@ -329,6 +325,12 @@
         <translation>(pa etiketë)</translation>
     </message>
     </context>
+<context>
+    <name>CreateWalletActivity</name>
+    </context>
+<context>
+    <name>CreateWalletDialog</name>
+    </context>
 <context>
     <name>EditAddressDialog</name>
     <message>
@@ -403,6 +405,9 @@
 <context>
     <name>OpenURIDialog</name>
     </context>
+<context>
+    <name>OpenWalletActivity</name>
+    </context>
 <context>
     <name>OptionsDialog</name>
     <message>
@@ -446,9 +451,6 @@
         <translation>i/e panjohur</translation>
     </message>
 </context>
-<context>
-    <name>QObject::QObject</name>
-    </context>
 <context>
     <name>QRImageWidget</name>
     </context>
@@ -508,7 +510,7 @@
         <source>Wallet</source>
         <translation>Portofol</translation>
     </message>
-    </context>
+</context>
 <context>
     <name>RecentRequestsTableModel</name>
     <message>
@@ -593,13 +595,6 @@
         <source>Pay To:</source>
         <translation>Paguaj drejt:</translation>
     </message>
-    <message>
-        <source>Enter a label for this address to add it to your address book</source>
-        <translation>Krijoni një etiketë për këtë adresë që t'ja shtoni librit të adresave</translation>
-    </message>
-</context>
-<context>
-    <name>SendConfirmationDialog</name>
     </context>
 <context>
     <name>ShutdownWindow</name>
@@ -619,13 +614,6 @@
         <translation>Alt+P</translation>
     </message>
     </context>
-<context>
-    <name>SplashScreen</name>
-    <message>
-        <source>[testnet]</source>
-        <translation>[testo rrjetin]</translation>
-    </message>
-</context>
 <context>
     <name>TrafficGraphWidget</name>
     </context>
@@ -786,14 +774,6 @@
     </context>
 <context>
     <name>bitcoin-core</name>
-    <message>
-        <source>Bitcoin Core</source>
-        <translation>Berthama Bitcoin</translation>
-    </message>
-    <message>
-        <source>Information</source>
-        <translation>Informacion</translation>
-    </message>
     <message>
         <source>Insufficient funds</source>
         <translation>Fonde te pamjaftueshme</translation>
@@ -802,9 +782,5 @@
         <source>Rescanning...</source>
         <translation>Rikerkim</translation>
     </message>
-    <message>
-        <source>Error</source>
-        <translation>Problem</translation>
-    </message>
-</context>
+    </context>
 </TS>
\ No newline at end of file
